(function () {
  'use strict';

  function copyStatics(to, from) {
    to.currentInput = from.currentInput;
    to.version = from.version;
    to.defaultProps = from.defaultProps;
    to.setDefaultProps = from.setDefaultProps;
    to.hideAll = from.hideAll;
    return to;
  }

  function enhance(tippyBase, propHOFs) {
    var tippyEnhanced = propHOFs.reduce(function (nextTippy, addProp) {
      return addProp(nextTippy);
    }, tippyBase);
    return copyStatics(tippyEnhanced, tippyBase);
  }

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  /**
   * Returns offsets for virtual reference when using a cursor point
   */

  function getVirtualOffsets(popper, isVerticalPlacement) {
    var size = isVerticalPlacement ? popper.offsetWidth : popper.offsetHeight;
    return {
      size: size,
      x: isVerticalPlacement ? size : 0,
      y: isVerticalPlacement ? 0 : size
    };
  }
  /**
   * Preserves the original function invocation when another function replaces it
   */

  function preserveInvocation(originalFn, currentFn, args) {
    if (originalFn && originalFn !== currentFn) {
      originalFn.apply(null, args);
    }
  }
  /**
   * Deletes properties from an object (pure)
   */

  function removeProperties(partialProps, keys) {
    var clone = _extends({}, partialProps);

    keys.forEach(function (key) {
      delete clone[key];
    });
    return clone;
  }
  /**
   * Works like Element.prototype.closest, but uses a callback instead
   */

  function closestCallback(element, callback) {
    while (element) {
      if (callback(element)) {
        return element;
      }

      element = element.parentElement;
    }

    return null;
  }
  /**
   * Determines if an array or string includes a string
   */

  function includes(a, b) {
    return a.indexOf(b) > -1;
  }

  /**
   * Returns the popper's placement, ignoring shifting (top-start, etc)
   */

  function getBasePlacement(placement) {
    return placement.split('-')[0];
  }

  var currentInput = {
    isTouch: false
  };

  function withFollowCursor(tippy) {
    return function (targets, optionalProps) {
      return tippy(targets, _extends({}, optionalProps, {
        onCreate: function onCreate(instance) {
          preserveInvocation(optionalProps && optionalProps.onCreate, instance.props.onCreate, [instance]);

          {
            instance.__extraProps__.followCursor = true;
          }

          if (instance.props.followCursor) {
            applyFollowCursor(instance);
          }
        }
      }));
    };
  }

  function applyFollowCursor(instance) {
    var reference = instance.reference;
    var originalSetProps = instance.setProps;
    var lastMouseMoveEvent;
    var isPopperInstanceCreated = false;
    var wasTriggeredByFocus = false;

    function addListener() {
      document.addEventListener('mousemove', onMouseMove);
    }

    function removeListener() {
      document.removeEventListener('mousemove', onMouseMove);
    }

    function onMouseMove(event) {
      if (wasTriggeredByFocus || !event) {
        return;
      }

      var _lastMouseMoveEvent = lastMouseMoveEvent = event,
          clientX = _lastMouseMoveEvent.clientX,
          clientY = _lastMouseMoveEvent.clientY;

      if (!instance.popperInstance || !isPopperInstanceCreated) {
        return;
      } // If the instance is interactive, avoid updating the position unless it's
      // over the reference element


      var isCursorOverReference = closestCallback(event.target, function (el) {
        return el === reference;
      });
      var rect = reference.getBoundingClientRect();
      var _ref = instance.props,
          followCursor = _ref.followCursor;
      var isHorizontal = followCursor === 'horizontal';
      var isVertical = followCursor === 'vertical';
      var isVerticalPlacement = includes(['top', 'bottom'], getBasePlacement(instance.state.currentPlacement)); // The virtual reference needs some size to prevent itself from overflowing

      var _getVirtualOffsets = getVirtualOffsets(instance.popper, isVerticalPlacement),
          size = _getVirtualOffsets.size,
          x = _getVirtualOffsets.x,
          y = _getVirtualOffsets.y;

      if (isCursorOverReference || !instance.props.interactive) {
        instance.popperInstance.reference = {
          // These `client` values don't get used by Popper.js if they are 0
          clientWidth: 0,
          clientHeight: 0,
          getBoundingClientRect: function getBoundingClientRect() {
            return {
              width: isVerticalPlacement ? size : 0,
              height: isVerticalPlacement ? 0 : size,
              top: (isHorizontal ? rect.top : clientY) - y,
              bottom: (isHorizontal ? rect.bottom : clientY) + y,
              left: (isVertical ? rect.left : clientX) - x,
              right: (isVertical ? rect.right : clientX) + x
            };
          }
        };
        instance.popperInstance.update();
      } // IIFE context vs. ESM/CJS context


      var ci = // @ts-ignore
      // eslint-disable-next-line
      typeof tippy !== 'undefined' ? tippy.currentInput : currentInput;

      if (ci.isTouch || followCursor === 'initial' && instance.state.isVisible) {
        removeListener();
      }
    }

    var _instance$props = instance.props,
        popperOptions = _instance$props.popperOptions,
        _onMount = _instance$props.onMount,
        _onTrigger = _instance$props.onTrigger,
        _onUntrigger = _instance$props.onUntrigger,
        _onHidden = _instance$props.onHidden; // Due to the virtual offsets normalization when using `followCursor`,
    // we need to use the opposite placement

    var placement = instance.props.placement;
    var normalizedPlacement = placement;

    function setNormalizedPlacement() {
      var shift = placement.split('-')[1];
      normalizedPlacement = instance.props.followCursor && shift ? placement.replace(shift, shift === 'start' ? 'end' : 'start') : placement;
      originalSetProps({
        placement: normalizedPlacement
      });
    }

    setNormalizedPlacement();

    function popperOnCreate(data) {
      preserveInvocation(popperOptions && popperOptions.onCreate, instance.props.popperOptions.onCreate, [data]);
      isPopperInstanceCreated = true;
    }

    instance.setProps({
      popperOptions: _extends({}, popperOptions, {
        onCreate: popperOnCreate
      }),
      onMount: function onMount(instance) {
        preserveInvocation(_onMount, instance.props.onMount, [instance]); // Popper's scroll listeners make sense for `true`, where the cursor
        // follows both axes. TODO: somehow keep scroll listeners for vertical
        // scrolling for "vertical", and horizontal scrolling for "horizontal".

        if (!wasTriggeredByFocus && instance.props.followCursor !== true) {
          instance.popperInstance.disableEventListeners();
        }

        onMouseMove(lastMouseMoveEvent);
      },
      onTrigger: function onTrigger(instance, event) {
        preserveInvocation(_onTrigger, instance.props.onTrigger, [instance, event]);
        wasTriggeredByFocus = event.type === 'focus';

        if (event instanceof MouseEvent) {
          lastMouseMoveEvent = event;
        }

        if (wasTriggeredByFocus && instance.popperInstance) {
          instance.popperInstance.reference = instance.reference;
        }

        if (!wasTriggeredByFocus && !(instance.state.isMounted && instance.props.followCursor === 'initial')) {
          addListener();
        }
      },
      onUntrigger: function onUntrigger(instance, event) {
        preserveInvocation(_onUntrigger, instance.props.onUntrigger, [instance, event]); // The listener gets added in `onTrigger()`, but due to potential delay(s)
        // the instance made be untriggered before it shows. `onHidden()` will
        // therefore never be invoked.

        if (!instance.state.isVisible) {
          removeListener();
        }
      },
      onHidden: function onHidden() {
        preserveInvocation(_onHidden, instance.props.onHidden, [instance]); // If scheduled to show before unmounting (e.g. delay: [500, 0]), the
        // listener should not be removed

        if (!instance.state.isScheduledToShow) {
          removeListener();
        }

        instance.popperInstance.options.placement = normalizedPlacement;
      }
    });

    instance.setProps = function (partialProps) {
      placement = partialProps.placement || placement;
      _onTrigger = partialProps.onTrigger || _onTrigger;
      _onUntrigger = partialProps.onUntrigger || _onUntrigger;
      _onMount = partialProps.onMount || _onMount;
      _onHidden = partialProps.onHidden || _onHidden;
      popperOptions = partialProps.popperOptions || popperOptions;
      originalSetProps(_extends({
        popperOptions: _extends({}, popperOptions, {
          onCreate: popperOnCreate
        })
      }, removeProperties(partialProps, ['placement', 'onTrigger', 'onUntrigger', 'onMount', 'onHidden', 'popperOptions'])));
      setNormalizedPlacement();
      onMouseMove(lastMouseMoveEvent);
    };
  }

  /* eslint-disable no-undef */

  if (typeof tippy === 'function') {
    window.tippy = enhance(tippy, [withFollowCursor]);
  } else {
    throw new Error('[tippy.js ERROR] `tippy` is not a global function. Make sure you have ' + 'included the tippy script before tippy-extra-props.');
  }

}());
//# sourceMappingURL=tippy-extra-props.js.map
